@@ -1,7 +1,6 @@
 import pytest
 import torch
 from magicsoup.containers import (
-    Protein,
     Molecule,
     CatalyticDomain,
     RegulatoryDomain,
@@ -93,15 +92,44 @@ def avg(*x):
     return sum(x) / len(x)
 
 
-def test_unsetting_cell_params():
-    Km = torch.randn(2, 3, 8)
-    Vmax = torch.randn(2, 3)
-    E = torch.randn(2, 3)
-    N = torch.randn(2, 3, 8)
-    A = torch.randn(2, 3, 8)
+def test_cell_params_with_catalytic_domains_and_co_factors():
+    # Dealing with stoichiometric numbers that cancel each other out
+    # in general, intermediate molecules should be 0 in N
+    # but the reaction must depend on abundance of the starting molecules
+    # the first domain defines these starting molecules
+    # if these are 0 in N, they must appear in A
 
-    cell_prots0 = [(0, i) for i in range(3)]
-    cell_prots1 = [(1, i) for i in range(3)]
+    # C0, P0:
+    # bc->d then d->2b so bc->2b, with N b=1 c=-1 d=0
+    # b needs to be added as activating effector A b=1
+    # C0, P1:
+    # d->2b then bc->d, with N b=1 c=-1 d=0
+    # d needs to be added as activating effector A d=1
+    #
+    # the stoichiometry in both proteins in C0 is the same, but
+    # the order in which domains were defined is different
+    # That's why they differ in A
+
+    # Domain spec indexes: (dom_types, reacts_trnspts_effctrs, Vmaxs, Kms, signs)
+    # fmt: off
+    c0 = [
+        [
+            (1, 2, 15, 1, 3), # catal, Vmax 1.2, Km 1.5, fwd, bc->d
+            (1, 1, 5, 1, 4), # catal, Vmax 1.1, Km 0.5, fwd, d->2b
+        ],
+        [
+            (1, 1, 5, 1, 4), # catal, Vmax 1.1, Km 0.5, fwd, d->2b
+            (1, 2, 15, 1, 3), # catal, Vmax 1.2, Km 1.5, fwd, bc->d
+        ]
+    ]
+
+    # fmt: on
+
+    Km = torch.zeros(1, 3, 8)
+    Vmax = torch.zeros(1, 3)
+    E = torch.zeros(1, 3)
+    N = torch.zeros(1, 3, 8)
+    A = torch.zeros(1, 3, 8)
 
     # test
     kinetics = get_kinetics()
@@ -110,13 +138,57 @@ def test_unsetting_cell_params():
     kinetics.E = E
     kinetics.N = N
     kinetics.A = A
-    kinetics.unset_cell_params(cell_prots=cell_prots0 + cell_prots1)
+    kinetics.set_cell_params(cell_idxs=[0], proteomes=[c0])
 
-    assert torch.all(Km == 0.0)
-    assert torch.all(Vmax == 0.0)
-    assert torch.all(E == 0.0)
-    assert torch.all(N == 0.0)
-    assert torch.all(A == 0.0)
+    # proteins in C0 only differ in A
+    for p in [0, 1]:
+        assert Km[0, p, 1] == pytest.approx(avg(1.5, 1 / 0.5), abs=TOLERANCE)
+        assert Km[0, p, 2] == pytest.approx(1.5, abs=TOLERANCE)
+        assert Km[0, p, 3] == pytest.approx(avg(0.5, 1 / 1.5), abs=TOLERANCE)
+        for i in [0, 4, 5, 6, 7]:
+            assert Km[0, p, i] == 0.0
+
+        assert Vmax[0, p] == pytest.approx(avg(1.1, 1.2), abs=TOLERANCE)
+
+        assert E[0, p] == 10 - 10
+
+        assert N[0, p, 1] == 1
+        assert N[0, p, 2] == -1
+        assert N[0, p, 3] == 0
+        for i in [0, 4, 5, 6, 7]:
+            assert N[0, p, i] == 0
+
+    assert A[0, 0, 1] == 1
+    assert A[0, 1, 3] == 1
+    assert A.sum() == 2
+
+    # test proteome representation
+
+    proteins = kinetics.get_proteome(proteome=c0)
+
+    p0 = proteins[0]
+    assert isinstance(p0.domains[0], CatalyticDomain)
+    assert p0.domains[0].substrates == [mb, mc]
+    assert p0.domains[0].products == [md]
+    assert p0.domains[0].vmax == pytest.approx(1.2, abs=TOLERANCE)
+    assert p0.domains[0].km == pytest.approx(1.5, abs=TOLERANCE)
+    assert isinstance(p0.domains[1], CatalyticDomain)
+    assert p0.domains[1].substrates == [md]
+    assert p0.domains[1].products == [mb, mb]
+    assert p0.domains[1].vmax == pytest.approx(1.1, abs=TOLERANCE)
+    assert p0.domains[1].km == pytest.approx(0.5, abs=TOLERANCE)
+
+    p1 = proteins[1]
+    assert isinstance(p1.domains[0], CatalyticDomain)
+    assert p1.domains[0].substrates == [md]
+    assert p1.domains[0].products == [mb, mb]
+    assert p1.domains[0].vmax == pytest.approx(1.1, abs=TOLERANCE)
+    assert p1.domains[0].km == pytest.approx(0.5, abs=TOLERANCE)
+    assert isinstance(p1.domains[1], CatalyticDomain)
+    assert p1.domains[1].substrates == [mb, mc]
+    assert p1.domains[1].products == [md]
+    assert p1.domains[1].vmax == pytest.approx(1.2, abs=TOLERANCE)
+    assert p1.domains[1].km == pytest.approx(1.5, abs=TOLERANCE)
 
 
 def test_cell_params_with_transporter_domains():
@@ -125,23 +197,23 @@ def test_cell_params_with_transporter_domains():
     # fmt: off
     c0 = [
         [
-            (2, 1, 5, 5, 1)  # transporter, mol a, Vmax 1.5, Km 0.5, fwd
+            (2, 5, 5, 1, 1)  # transporter, Vmax 1.5, Km 0.5, fwd, mol a
         ],
         [
-            (2, 1, 5, 5, 1), # transporter, mol a, Vmax 1.5, Km 0.5, fwd
-            (2, 1, 1, 2, 2)  # transporter, mol a, Vmax 1.1, Km 0.2, bwd
+            (2, 5, 5, 1, 1), # transporter, Vmax 1.5, Km 0.5, fwd, mol a
+            (2, 1, 2, 2, 1)  # transporter, Vmax 1.1, Km 0.2, bwd, mol a
         ],
     ]
     c1 = [
         [
-            (2, 1, 5, 4, 1), # transporter, mol a, Vmax 1.5, Km 0.4, fwd
-            (2, 1, 4, 5, 1), # transporter, mol a, Vmax 1.4, Km 0.5, fwd
-            (2, 2, 3, 6, 1), # transporter, mol b, Vmax 1.3, Km 0.6, fwd
-            (2, 3, 2, 7, 1)  # transporter, mol c, Vmax 1.2, Km 0.7, fwd
+            (2, 5, 4, 1, 1), # transporter, Vmax 1.5, Km 0.4, fwd, mol a
+            (2, 4, 5, 1, 1), # transporter, Vmax 1.4, Km 0.5, fwd, mol a
+            (2, 3, 6, 1, 2), # transporter, Vmax 1.3, Km 0.6, fwd, mol b
+            (2, 2, 7, 1, 3)  # transporter, Vmax 1.2, Km 0.7, fwd, mol c
         ],
         [
-            (1, 1, 10, 5, 1), # catal, a->b, Vmax 2.0, Km 0.5, fwd
-            (2, 1, 5, 5, 1)   # transporter, mol a, Vmax 1.5, Km 0.5, fwd
+            (1, 10, 5, 1, 1), # catal, Vmax 2.0, Km 0.5, fwd, a->b
+            (2, 5, 5, 1, 1)   # transporter, Vmax 1.5, Km 0.5, fwd, mol a
         ],
     ]
     # fmt: on
@@ -223,7 +295,10 @@ def test_cell_params_with_transporter_domains():
     for i in [2, 3, 5, 6, 7]:
         assert N[1, 1, i] == 0
 
-    assert not A.any()
+    # a was imported and exported, so its N=0
+    # but it must be added as effector
+    assert A[0, 1, 0] == 1
+    assert A.sum() == 1
 
     # test proteome representation
 
@@ -283,27 +358,27 @@ def test_cell_params_with_regulatory_domains():
     # fmt: off
     c0 = [
         [
-            (1, 1, 10, 5, 1), # catal, a->b, Vmax 2.0, Km 0.5, fwd
-            (3, 3, 0, 10, 1), # reg, c, Km 1.0, cyto, act
-            (3, 4, 0, 20, 2), # reg, d, Km 2.0, cyto, inh
+            (1, 10, 5, 1, 1), # catal, Vmax 2.0, Km 0.5, fwd, a->b
+            (3, 0, 10, 1, 3), # reg, Km 1.0, cyto, act, c
+            (3, 0, 20, 2, 4), # reg, Km 2.0, cyto, inh, d
         ],
         [
-            (1, 1, 10, 5, 1), # catal, a->b, Vmax 2.0, Km 0.5, fwd
-            (3, 1, 0, 10, 1), # reg, a, Km 1.0, cyto, act
-            (3, 5, 0, 15, 1), # reg, a, Km 1.5, transm, act
+            (1, 10, 5, 1, 1), # catal, Vmax 2.0, Km 0.5, fwd, a->b
+            (3, 0, 10, 1, 1), # reg, Km 1.0, cyto, act, a
+            (3, 0, 15, 1, 5), # reg, Km 1.5, transm, act, a
         ]
     ]
 
     c1 = [
         [
-            (1, 1, 10, 5, 1), # catal, a->b, Vmax 2.0, Km 0.5, fwd
-            (3, 2, 0, 10, 2), # reg, b, Km 1.0, cyto, inh
-            (3, 6, 0, 15, 2), # reg, b, Km 1.5, transm, inh
+            (1, 10, 5, 1, 1), # catal, Vmax 2.0, Km 0.5, fwd, a->b
+            (3, 0, 10, 2, 2), # reg, Km 1.0, cyto, inh, b
+            (3, 0, 15, 2, 6), # reg, Km 1.5, transm, inh, b
         ],
         [
-            (1, 1, 10, 5, 1), # catal, a->b, Vmax 2.0, Km 0.5, fwd
-            (3, 4, 0, 10, 1), # reg, d, Km 1.0, cyto, act
-            (3, 4, 0, 15, 1), # reg, d, Km 1.5, cyto, act
+            (1, 10, 5, 1, 1), # catal, Vmax 2.0, Km 0.5, fwd, a->b
+            (3, 0, 10, 1, 4), # reg, Km 1.0, cyto, act, d
+            (3, 0, 15, 1, 4), # reg, Km 1.5, cyto, act, d
         ]
     ]
     # fmt: on
@@ -378,7 +453,7 @@ def test_cell_params_with_regulatory_domains():
     for i in [4, 5, 6, 7]:
         assert N[0, 0, i] == 0
     assert N[0, 1, 0] == -1
-    assert N[0, 1, 1] == 1  # b is added and removed
+    assert N[0, 1, 1] == 1
     assert N[0, 1, 2] == 0
     assert N[0, 1, 3] == 0
     for i in [4, 5, 6, 7]:
@@ -387,14 +462,14 @@ def test_cell_params_with_regulatory_domains():
         assert N[0, 2, i] == 0
 
     assert N[1, 0, 0] == -1
-    assert N[1, 0, 1] == 1  # b is added and removed
+    assert N[1, 0, 1] == 1
     assert N[1, 0, 2] == 0
     assert N[1, 0, 3] == 0
     for i in [4, 5, 6, 7]:
         assert N[1, 0, i] == 0
     assert N[1, 1, 0] == -1
     assert N[1, 1, 1] == 1
-    assert N[1, 1, 2] == 0  # c is added and removed
+    assert N[1, 1, 2] == 0
     assert N[1, 1, 3] == 0
     for i in [4, 5, 6, 7]:
         assert N[1, 1, i] == 0
@@ -514,25 +589,25 @@ def test_cell_params_with_catalytic_domains():
     # fmt: off
     c0 = [
         [
-            (1, 1, 1, 5, 1), # catal, a->b, Vmax 1.1, Km 0.5, fwd
-            (1, 3, 2, 15, 2), # catal, bc->d, Vmax 1.2, Km 1.5, bwd
+            (1, 1, 5, 1, 1), # catal, Vmax 1.1, Km 0.5, fwd, a->b
+            (1, 2, 15, 2, 3), # catal, Vmax 1.2, Km 1.5, bwd, bc->d
         ],
         [
-            (1, 2, 10, 9, 1), # catal, b->c, Vmax 2.0, Km 0.9, fwd
-            (1, 3, 3, 12, 2), # catal, bc->d, Vmax 1.3, Km 1.2, bwd
+            (1, 10, 9, 1, 2), # catal, Vmax 2.0, Km 0.9, fwd, b->c
+            (1, 3, 12, 2, 3), # catal, Vmax 1.3, Km 1.2, bwd, bc->d
         ],
         [
-            (1, 4, 19, 29, 1), # catal, d->bb, Vmax 2.9, Km 2.9, fwd
+            (1, 19, 29, 1, 4), # catal, Vmax 2.9, Km 2.9, fwd, d->bb
         ]
     ]
     c1 = [
         [
-            (1, 1, 1, 3, 2), # catal, a->b, Vmax 1.1, Km 0.3, bwd
-            (1, 3, 11, 14, 2), # catal, bc->d, Vmax 2.1, Km 1.4, bwd
+            (1, 1, 3, 2, 1), # catal, Vmax 1.1, Km 0.3, bwd, a->b
+            (1, 11, 14, 2, 3), # catal, Vmax 2.1, Km 1.4, bwd, bc->d
         ],
         [
-            (1, 2, 9, 3, 1), # catal, b->c, Vmax 1.9, Km 0.3, fwd
-            (1, 3, 13, 17, 1), # catal, bc->d, Vmax 2.3, Km 1.7, fwd
+            (1, 9, 3, 1, 2), # catal, Vmax 1.9, Km 0.3, fwd, b->c
+            (1, 13, 17, 1, 3), # catal, Vmax 2.3, Km 1.7, fwd, bc->d
         ]
     ]
 
@@ -637,7 +712,12 @@ def test_cell_params_with_catalytic_domains():
     for i in range(8):
         assert N[1, 2, i] == 0
 
-    assert not A.any()
+    # b was a reactant in the first domain,
+    # but was then created again yielding N b=0
+    # it thus has to be added as activating effector
+    assert A[0, 1, 1] == 1
+    assert A[1, 0, 1] == 1
+    assert A.sum() == 2
 
     # test protein representation
 
